@@ -75,6 +75,14 @@ def test_4d_beams(data_sdav_beams, use_dask):
     np.testing.assert_almost_equal(c.beams[0].minor.value, 0.1)
 
 
+def test_4d_beams_nounits(data_sdav_beams_nounits, use_dask):
+    c = SpectralCube.read(data_sdav_beams_nounits, use_dask=use_dask)
+    np.testing.assert_almost_equal(c.beams[0].major.value, 0.4)
+    np.testing.assert_almost_equal(c.beams[0].minor.value, 0.1)
+    assert c.beams[0].major.unit == u.arcsec
+    assert c.beams[0].minor.unit == u.arcsec
+
+
 def test_3d_beams_roundtrip(tmpdir, data_vda_beams, use_dask):
     c = SpectralCube.read(data_vda_beams, use_dask=use_dask)
     np.testing.assert_almost_equal(c.beams[0].major.value, 0.4)
@@ -88,6 +96,8 @@ def test_3d_beams_roundtrip(tmpdir, data_vda_beams, use_dask):
 
     np.testing.assert_almost_equal(c2.beams[0].major.value, 0.4)
     np.testing.assert_almost_equal(c2.beams[0].minor.value, 0.1)
+    assert c2.beams[0].major.unit == u.arcsec
+    assert c2.beams[0].minor.unit == u.arcsec
 
 
 def test_4d_beams_roundtrip(tmpdir, data_sdav_beams, use_dask):
@@ -104,6 +114,9 @@ def test_4d_beams_roundtrip(tmpdir, data_sdav_beams, use_dask):
 
     np.testing.assert_almost_equal(c2.beams[0].major.value, 0.4)
     np.testing.assert_almost_equal(c2.beams[0].minor.value, 0.1)
+    assert c2.beams[0].major.unit == u.arcsec
+    assert c2.beams[0].minor.unit == u.arcsec
+
 
 def test_1d(data_5_spectral):
     hdul = pyfits.open(data_5_spectral)
@@ -134,3 +147,15 @@ def test_aips_beams_units(tmpdir, data_455_degree_beams, use_dask):
     np.testing.assert_almost_equal(c.beams[0].minor.value, 0.1/3600)
     np.testing.assert_almost_equal(c.beams[0].major.to(u.arcsec).value, 0.4)
     np.testing.assert_almost_equal(c.beams[0].minor.to(u.arcsec).value, 0.1)
+
+
+
+def test_vrsc_fullstokes_read_fits(data_advs_beams_fullstokes):
+    '''
+    Test reading a spectral line data cube with full stokes and a beam table.
+    '''
+    c = StokesSpectralCube.read(data_advs_beams_fullstokes)
+
+    for component in ['I', 'Q', 'U', 'V']:
+        assert isinstance(c[component], VaryingResolutionSpectralCube)
+        assert hasattr(c[component], 'beams')
