@@ -2,8 +2,9 @@ import sys
 import pytest
 import tempfile
 import numpy as np
+import os
 
-from astropy import units as u
+from astropy import constants, units as u
 from astropy import convolution
 from astropy.wcs import WCS
 from astropy import wcs
@@ -24,7 +25,9 @@ NPY_VERSION_CHECK = LooseVersion(np.version.version) >= "1.13"
 from radio_beam import beam, Beam
 
 from .. import SpectralCube
+from ..masks import BooleanArrayMask
 from ..utils import WCSCelestialError
+from ..cube_utils import mosaic_cubes, combine_headers
 from .test_spectral_cube import cube_and_raw
 from .test_projection import load_projection
 from . import path, utilities
@@ -32,9 +35,12 @@ from . import path, utilities
 WINDOWS = sys.platform == "win32"
 
 
-def test_convolution(data_255_delta, use_dask):
+@pytest.mark.parametrize('allow_huge_operations', (True, False))
+def test_convolution(data_255_delta, allow_huge_operations, use_dask):
     cube, data = cube_and_raw(data_255_delta, use_dask=use_dask)
 
+    cube.allow_huge_operations = allow_huge_operations
+
     # 1" convolved with 1.5" -> 1.8027....
     target_beam = Beam(1.802775637731995*u.arcsec, 1.802775637731995*u.arcsec,
                        0*u.deg)
@@ -56,9 +62,12 @@ def test_convolution(data_255_delta, use_dask):
     assert np.all(conv_cube.filled_data[1,:,:] == 0.0)
 
 
-def test_beams_convolution(data_455_delta_beams, use_dask):
+@pytest.mark.parametrize('allow_huge_operations', (True, False))
+def test_beams_convolution(data_455_delta_beams, allow_huge_operations, use_dask):
     cube, data = cube_and_raw(data_455_delta_beams, use_dask=use_dask)
 
+    cube.allow_huge_operations = allow_huge_operations
+
     # 1" convolved with 1.5" -> 1.8027....
     target_beam = Beam(1.802775637731995*u.arcsec, 1.802775637731995*u.arcsec,
                        0*u.deg)
@@ -106,6 +115,11 @@ def test_reproject(use_memmap, data_adv, use_dask):
     wcs_out.wcs.crval = [134.37608, -31.939241, wcs_in.wcs.crval[2]]
     wcs_out.wcs.crpix = [2., 2., wcs_in.wcs.crpix[2]]
 
+    # cube is doppler-optical by default, which uses the rest wavelength,
+    # which isn't auto-computed, resulting in nan pixels in the WCS transform
+    wcs_out.wcs.restwav = 0.21106114549833
+    cube._wcs.wcs.restwav = 0.21106114549833
+
     header_out = cube.header
     header_out['NAXIS1'] = 4
     header_out['NAXIS2'] = 5
@@ -115,29 +129,52 @@ def test_reproject(use_memmap, data_adv, use_dask):
     result = cube.reproject(header_out, use_memmap=use_memmap)
 
     assert result.shape == (cube.shape[0], 5, 4)
+    
+    # empirically, this is how close we can get after https://github.com/astropy/astropy/pull/14508
+    tolerance = 1e-12
+
+    assert wcs_out.wcs.compare(WCS(header_out).wcs, tolerance=tolerance)
     # Check WCS in reprojected matches wcs_out
-    assert wcs_out.wcs.compare(result.wcs.wcs)
+    assert wcs_out.wcs.compare(result.wcs.wcs, tolerance=tolerance)
     # And that the headers have equivalent WCS info.
     result_wcs_from_header = WCS(result.header)
-    assert result_wcs_from_header.wcs.compare(wcs_out.wcs)
+    assert result_wcs_from_header.wcs.compare(wcs_out.wcs, tolerance=tolerance)
 
 
 def test_spectral_smooth(data_522_delta, use_dask):
 
     cube, data = cube_and_raw(data_522_delta, use_dask=use_dask)
 
-    result = cube.spectral_smooth(kernel=convolution.Gaussian1DKernel(1.0), use_memmap=False)
+    kernel = convolution.Gaussian1DKernel(1.0)
+
+    result = cube.spectral_smooth(kernel=kernel, use_memmap=False)
+
+    # check that all values come out right from the cube creation
+    np.testing.assert_almost_equal(cube[2,:,:].value, 1.0)
+    np.testing.assert_almost_equal(cube.unitless_filled_data[:2,:,:], 0.0)
+    np.testing.assert_almost_equal(cube.unitless_filled_data[3:,:,:], 0.0)
+
+    # make sure the kernel comes out right; the convolution test will fail if this is wrong
+    assert kernel.array.size == 9
+    # this was the old astropy normalization
+    # We don't actually need the kernel to match these values, but I'm leaving this here
+    # as a note for future us:
+    # https://github.com/astropy/astropy/pull/13299
+    # the error came about because we were using two different kernel sizes, which resulted in
+    # two different normalizations after the correction in 13299
+    # Before 13299, normalization was not guaranteed.
+    #np.testing.assert_almost_equal(kernel.array[2:-2],
+    #                               np.array([0.05399097, 0.24197072, 0.39894228, 0.24197072, 0.05399097]))
 
     np.testing.assert_almost_equal(result[:,0,0].value,
-                                   convolution.Gaussian1DKernel(1.0,
-                                                                x_size=5).array,
+                                   kernel.array[2:-2],
                                    4)
 
-    result = cube.spectral_smooth(kernel=convolution.Gaussian1DKernel(1.0), use_memmap=True)
+    # second test with memmap=True
+    result = cube.spectral_smooth(kernel=kernel, use_memmap=True)
 
     np.testing.assert_almost_equal(result[:,0,0].value,
-                                   convolution.Gaussian1DKernel(1.0,
-                                                                x_size=5).array,
+                                   kernel.array[2:-2],
                                    4)
 
 def test_catch_kernel_with_units(data_522_delta, use_dask):
@@ -158,19 +195,19 @@ def test_spectral_smooth_4cores(data_522_delta):
 
     cube, data = cube_and_raw(data_522_delta, use_dask=False)
 
-    result = cube.spectral_smooth(kernel=convolution.Gaussian1DKernel(1.0), num_cores=4, use_memmap=True)
+    kernel = convolution.Gaussian1DKernel(1.0)
+    result = cube.spectral_smooth(kernel=kernel, num_cores=4, use_memmap=True)
 
+    assert kernel.array.size == 9
     np.testing.assert_almost_equal(result[:,0,0].value,
-                                   convolution.Gaussian1DKernel(1.0,
-                                                                x_size=5).array,
+                                   kernel.array[2:-2],
                                    4)
 
     # this is one way to test non-parallel mode
-    result = cube.spectral_smooth(kernel=convolution.Gaussian1DKernel(1.0), num_cores=4, use_memmap=False)
+    result = cube.spectral_smooth(kernel=kernel, num_cores=4, use_memmap=False)
 
     np.testing.assert_almost_equal(result[:,0,0].value,
-                                   convolution.Gaussian1DKernel(1.0,
-                                                                x_size=5).array,
+                                   kernel.array[2:-2],
                                    4)
 
     # num_cores = 4 is a contradiction with parallel=False, so we want to make
@@ -180,12 +217,11 @@ def test_spectral_smooth_4cores(data_522_delta):
                                  "multiple cores were: these are incompatible "
                                  "options.  Either specify num_cores=1 or "
                                  "parallel=True")):
-        result = cube.spectral_smooth(kernel=convolution.Gaussian1DKernel(1.0),
+        result = cube.spectral_smooth(kernel=kernel,
                                       num_cores=4, parallel=False)
 
     np.testing.assert_almost_equal(result[:,0,0].value,
-                                   convolution.Gaussian1DKernel(1.0,
-                                                                x_size=5).array,
+                                   kernel.array[2:-2],
                                    4)
 
 
@@ -218,6 +254,47 @@ def test_spectral_interpolate(data_522_delta, use_dask):
     assert cube.wcs.wcs.compare(orig_wcs.wcs)
 
 
+def test_spectral_interpolate_varying_chunksize(data_255_delta):
+
+    cube, data = cube_and_raw(data_255_delta, use_dask=True)
+
+    orig_wcs = cube.wcs.deepcopy()
+
+    # midpoint between each position
+    sg = (cube.spectral_axis[1:] + cube.spectral_axis[:-1])/2.
+
+    # Force unequal chunks
+    cube = cube.rechunk((-1, 2, 2))
+
+    result = cube.spectral_interpolate(spectral_grid=sg, force_rechunk=False)
+
+    np.testing.assert_almost_equal(result[:,2,2].value,
+                                   [0.5])
+
+    assert cube.wcs.wcs.compare(orig_wcs.wcs)
+
+    # Ensure the spatial chunk sizes vary
+    assert cube._data.chunks[1] == (2, 2, 1)
+    assert result._data.chunks[1] == (2, 2, 1)
+
+
+def test_spectral_interpolate_rechunk_fail(data_255_delta):
+
+    cube, data = cube_and_raw(data_255_delta, use_dask=True)
+
+    orig_wcs = cube.wcs.deepcopy()
+
+    # midpoint between each position
+    sg = (cube.spectral_axis[1:] + cube.spectral_axis[:-1])/2.
+
+    # Force >1 chunk in spectral dimension
+    cube = cube.rechunk((1, -1, -1))
+
+    with pytest.raises(ValueError,
+                       match=("The cube currently has 2 chunks along")):
+        cube.spectral_interpolate(spectral_grid=sg, force_rechunk=False)
+
+
 def test_spectral_interpolate_with_fillvalue(data_522_delta, use_dask):
 
     cube, data = cube_and_raw(data_522_delta, use_dask=use_dask)
@@ -520,3 +597,44 @@ def test_reproject_3D_memory():
 
     assert result.wcs.wcs.crval[0] == 0.001
     assert result.wcs.wcs.crpix[0] == 2.
+
+@pytest.mark.parametrize('spectral_block_size,use_memmap', ((None, False),
+                                                            (100, False),
+                                                            (None, True),
+                                                            (100, False),
+                                                            (1, True),
+                                                            (1, False),
+                                                            ))
+def test_mosaic_cubes(use_memmap, data_adv, use_dask, spectral_block_size):
+
+    pytest.importorskip('reproject')
+
+    # Read in data to use
+    cube, data = cube_and_raw(data_adv, use_dask=use_dask)
+
+    # cube is doppler-optical by default, which uses the rest wavelength,
+    # which isn't auto-computed, resulting in nan pixels in the WCS transform
+    cube._wcs.wcs.restwav = constants.c.to(u.m/u.s).value / cube.wcs.wcs.restfrq
+
+    expected_wcs = WCS(combine_headers(cube.header, cube.header)).celestial
+
+    # Make two overlapping cubes of the data
+    part1 = cube[:, :round(cube.shape[1]*2./3.), :]
+    part2 = cube[:, round(cube.shape[1]/3.):, :]
+
+    assert part1.wcs.wcs.restwav != 0
+    assert part2.wcs.wcs.restwav != 0
+
+    result = mosaic_cubes([part1, part2], order='nearest-neighbor',
+                          roundtrip_coords=False,
+                          spectral_block_size=spectral_block_size)
+
+    # Check that the shapes are the same
+    assert result.shape == cube.shape
+
+    # Check WCS in reprojected matches wcs_out
+    # (comparing WCS failed for no reason we could discern)
+    assert repr(expected_wcs) == repr(result.wcs.celestial)
+    # Check that values of original and result are comparable
+    np.testing.assert_almost_equal(result.filled_data[:].value, cube.filled_data[:].value, decimal=3)
+    # only good to 3 decimal places is not amazing...
